import { ParsingState } from "./state";
import { ReplyKind } from "../../reply";
import { AnyParserAction } from "../action";
/**
 * Basic implementation of the ParsingState interface.
 */
export declare class BasicParsingState implements ParsingState {
    input: string;
    position: number;
    stack: any[];
    initialUserState: any;
    userState: any;
    value: any;
    kind: ReplyKind;
    expecting: string;
    constructor(input: string);
    readonly isOk: boolean;
    readonly isSoft: boolean;
    readonly isHard: boolean;
    readonly isFatal: boolean;
    atLeast(kind: ReplyKind): boolean;
    atMost(kind: ReplyKind): boolean;
}
/**
 * A parsing action to perform. A parsing action is a fundamental operation that mutates a ParsingState.
 */
export declare abstract class ParjsAction implements AnyParserAction {
    abstract expecting: string;
    displayName: string;
    /**
     * Whether this action returns a value or not. Determines if the parser is loud or not.
     */
    abstract isLoud: boolean;
    /**
     * Perform the action on the given ParsingState. This is a wrapper around a derived action's _apply method.
     * @param ps The parsing state.
     */
    apply(ps: ParsingState): void;
    /**
     * The internal operation performed by the action. This will be overriden by derived classes.
     * @param ps
     * @private
     */
    protected abstract _apply(ps: ParsingState): void | void;
}
/**
 * Inherited by parser actions for basic parsers (e.g. string or numeric parsers), rather than combinators.
 */
export declare abstract class ParjsBasicAction extends ParjsAction {
    isLoud: boolean;
}
