"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @module parjs/internal/implementation/combinators
 */
/** */
const action_1 = require("../../action");
const issues_1 = require("../../issues");
const reply_1 = require("../../../../reply");
class PrsAlternatives extends action_1.ParjsAction {
    constructor(_alts) {
        super();
        this._alts = _alts;
        //if the list is empty, every won't execute and alts[0] won't be called.
        if (!_alts.every(x => x.isLoud === _alts[0].isLoud)) {
            issues_1.Issues.mixedLoudnessNotPermitted("alts");
        }
        _alts.length === 0 && issues_1.Issues.willAlwaysFail("alts");
        this.isLoud = _alts[0].isLoud;
        this.expecting = `any of: ${_alts.join(", ")}`;
    }
    _apply(ps) {
        let { position } = ps;
        let { _alts } = this;
        for (let i = 0; i < _alts.length; i++) {
            //go over each alternative.
            let cur = _alts[i];
            //apply it on the current state.
            cur.apply(ps);
            if (ps.isOk) {
                //if success, return. The PS records the result.
                return;
            }
            else if (ps.isSoft) {
                //backtrack to the original position and try again.
                ps.position = position;
            }
            else {
                //if failure, return false,
                return;
            }
        }
        ps.kind = reply_1.ReplyKind.SoftFail;
    }
}
exports.PrsAlternatives = PrsAlternatives;

//# sourceMappingURL=alts.js.map