/**
 * @module parjs
 */
/** */
import { AnyParserAction, Trace } from "./internal";
/**
 * Indicates a success reply and contains the value and other information.
 */
export declare class SuccessReply<T> {
    value: T;
    kind: "OK";
    constructor(value: T);
    toString(): string;
}
export interface ErrorLocation {
    row: number;
    column: number;
}
/**
 * An object indicating trace information about the state of parsing when it was stopped.
 */
export interface Trace {
    userState: object;
    position: number;
    reason: string;
    kind: ReplyKind.Fail;
    location: ErrorLocation;
    stackTrace: AnyParserAction[];
    input: string;
}
export declare class FailureReply {
    trace: Trace;
    reason: string;
    constructor(trace: Trace);
    readonly value: never;
    readonly kind: ReplyKind.Fail;
    toString(): string;
}
/**
 * A type that represents a SuccessReply or a FailureReply. Returned by parsers.
 */
export declare type Reply<T> = (SuccessReply<T> | FailureReply);
/**
 * A type representing the reply of a quiet parser.
 */
export declare type QuietReply = Reply<void>;
/**
 * Namespace that contains the different reply kinds/error levels.
 */
export declare namespace ReplyKind {
    /**
     * An Unknown reply. Used internally.
     */
    type Unknown = "Unknown";
    /**
     * The OK reply type.
     */
    type Ok = "OK";
    /**
     * The soft failure type.
     */
    type SoftFail = "Soft";
    /**
     * The hard failure type.
     */
    type HardFail = "Hard";
    /**
     * The fatal failure type.
     */
    type FatalFail = "Fatal";
    /**
     * An Unknown reply.
     */
    const Unknown: Unknown;
    /**
     * An OK reply.
     */
    const Ok: Ok;
    /**
     * A soft failure reply.
     */
    const SoftFail: SoftFail;
    /**
     * A hard failure reply.
     */
    const HardFail: HardFail;
    /**
     * A fatal failure reply.
     */
    const FatalFail: FatalFail;
    /**
     * Specifies any kind of failure.
     */
    type Fail = HardFail | FatalFail | SoftFail;
}
/**
 * Specifies a reply kind, indicating success or failure, and the severity of the failure.
 */
export declare type ReplyKind = ReplyKind.Ok | ReplyKind.Fail | ReplyKind.Unknown;
